import numpy as np
from tqdm import tqdm

import gym
from stable_baselines3 import A2C
from stable_baselines3.common.env_util import make_vec_env
import matplotlib.pyplot as plt


if __name__ == "__main__":
    episodes = 500

    # Create and wrap the CartPole environment
    env = make_vec_env("CartPole-v1", n_envs=4)
    
    # Define the A2C model
    model = A2C('MlpPolicy', env, verbose=1)

    # Training loop
    episode_rewards = []
    for episode in tqdm(range(episodes)):
        # Reinitialize environment and rewards for each episode
        obs = env.reset()
        episode_reward = 0

        while True:
            # Compute action probabilities
            action, _states = model.predict(obs)
            obs, reward, done, _ = env.step(action)
            episode_reward += np.sum(reward)

            # Break if any environment is done
            if np.any(done):
                break
        
        episode_rewards.append(episode_reward)

    # Save model
    model.save("a2c_cartpole_model")

    # Close environments
    env.close()
    
    # Plot result
    plt.plot(episode_rewards)
    plt.xlabel('Episode')
    plt.ylabel('Total reward')
    plt.title('Evolution of reward')
    plt.show()

