import gym
import numpy as np

from stable_baselines3 import A2C
from stable_baselines3.common.env_util import make_vec_env

from stable_baselines3.common.callbacks import EvalCallback
import wandb
from wandb.integration.sb3 import WandbCallback


if __name__ == "__main__":
    # Configuration for W&B
    wandb.init(project="wandb_test_cartpole", name="cartpole_v1")

    # Create environnement
    env = make_vec_env("CartPole-v1", n_envs=4)

    # Model initialization
    model = A2C("MlpPolicy", env, verbose=1, tensorboard_log="./wandb_test_cartpole_tensorboard/")

    # Evaluation callback
    eval_callback = EvalCallback(
        env,
        best_model_save_path='./logs/',
        log_path='./logs/',
        eval_freq=500,
        deterministic=True,
        render=False
    )

    # Weights & Biases callback
    wb_callback = WandbCallback(
        gradient_save_freq=1000,
        model_save_path=f"models/{wandb.run.id}",
        verbose=2,
    )

    # Actual training
    model.learn(total_timesteps=25000, callback=[eval_callback, wb_callback])
    model.save("test_wb_cartpole")
    wandb.finish()